@@ -1,33 +1,46 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: May 27, 2015
+# Last Change: October 22, 2015
 # URL: https://humanfriendly.readthedocs.org
 
+"""The main module of the `humanfriendly` package."""
+
 # Semi-standard module versioning.
-__version__ = '1.25'
+__version__ = '1.37'
 
 # Standard library modules.
-import collections
-import math
 import multiprocessing
+import numbers
 import os
 import os.path
 import re
 import sys
-import textwrap
 import time
 
+# In humanfriendly 1.23 the format_table() function was added to render a table
+# using characters like dashes and vertical bars to emulate borders. Since then
+# support for other tables has been added and the name of format_table() has
+# changed. The following import statement preserves backwards compatibility.
+from humanfriendly.tables import format_pretty_table as format_table  # NOQA
+
+# In humanfriendly 1.30 the following text manipulation functions were moved
+# out into a separate module to enable their usage in other modules of the
+# humanfriendly package (without causing circular imports).
+from humanfriendly.text import (  # NOQA
+    compact, concatenate, dedent, format, is_empty_line,
+    pluralize, tokenize, trim_empty_lines,
+)
+
+# Compatibility with Python 2 and 3.
 try:
     # Python 2.
     interactive_prompt = raw_input
     string_types = basestring
-    unicode_type = unicode
 except NameError:
     # Python 3.
     interactive_prompt = input
     string_types = str
-    unicode_type = str
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -48,137 +61,22 @@ disk_size_units = (dict(prefix='b', divider=1, singular='byte', plural='bytes'),
                    dict(prefix='t', divider=1024**4, singular='TB', plural='TB'),
                    dict(prefix='p', divider=1024**5, singular='PB', plural='PB'))
 
+# Common length size units, used for formatting and parsing.
+length_size_units = (dict(prefix='nm', divider=1e-09, singular='nm', plural='nm'),
+                     dict(prefix='mm', divider=1e-03, singular='mm', plural='mm'),
+                     dict(prefix='cm', divider=1e-02, singular='cm', plural='cm'),
+                     dict(prefix='m', divider=1, singular='metre', plural='metres'),
+                     dict(prefix='km', divider=1000, singular='km', plural='km'))
+
 # Common time units, used for formatting of time spans.
-time_units = (dict(divider=1, singular='second', plural='seconds'),
-              dict(divider=60, singular='minute', plural='minutes'),
-              dict(divider=60*60, singular='hour', plural='hours'),
-              dict(divider=60*60*24, singular='day', plural='days'),
-              dict(divider=60*60*24*7, singular='week', plural='weeks'),
-              dict(divider=60*60*24*7*52, singular='year', plural='years'))
+time_units = (dict(divider=1e-3, singular='millisecond', plural='milliseconds', abbreviation='ms'),
+              dict(divider=1, singular='second', plural='seconds', abbreviation='s'),
+              dict(divider=60, singular='minute', plural='minutes', abbreviation='m'),
+              dict(divider=60*60, singular='hour', plural='hours', abbreviation='h'),
+              dict(divider=60*60*24, singular='day', plural='days', abbreviation='d'),
+              dict(divider=60*60*24*7, singular='week', plural='weeks', abbreviation='w'),
+              dict(divider=60*60*24*7*52, singular='year', plural='years', abbreviation='y'))
 
-def compact(text, *args, **kw):
-    """
-    Compact whitespace in a string.
-
-    Trims leading and trailing whitespace, replaces runs of whitespace
-    characters with a single space and interpolates any arguments (using
-    :func:`format()`).
-
-    :param text: The text to compact (a string).
-    :param args: Any positional arguments are interpolated using :func:`format()`.
-    :param kw: Any keyword arguments are interpolated using :func:`format()`.
-    :returns: The compacted text (a string).
-    """
-    non_whitespace_tokens = text.split()
-    compacted_text = ' '.join(non_whitespace_tokens)
-    return format(compacted_text, *args, **kw)
-
-def dedent(text, *args, **kw):
-    """
-    Dedent a string (remove common leading whitespace from all lines).
-
-    Removes common leading whitespace from all lines in the string (using
-    :func:`textwrap.dedent()`), removes leading and trailing empty lines (using
-    :func:`trim_empty_lines()`) and interpolates any arguments (using
-    :func:`format()`).
-
-    :param text: The text to dedent (a string).
-    :param args: Any positional arguments are interpolated using :func:`format()`.
-    :param kw: Any keyword arguments are interpolated using :func:`format()`.
-    :returns: The dedented text (a string).
-    """
-    dedented_text = textwrap.dedent(text)
-    trimmed_text = trim_empty_lines(dedented_text)
-    return format(trimmed_text, *args, **kw)
-
-def trim_empty_lines(text):
-    """
-    Trim leading and trailing empty lines from the given text.
-
-    :param text: The text to trim (a string).
-    :returns: The trimmed text (a string).
-    """
-    lines = text.splitlines(True)
-    while lines and is_empty_line(lines[0]):
-        lines.pop(0)
-    while lines and is_empty_line(lines[-1]):
-        lines.pop(-1)
-    return ''.join(lines)
-
-def is_empty_line(text):
-    """
-    Check if a text is empty or contains only whitespace.
-
-    :param text: The text to check for "emptiness" (a string).
-    :returns: :data:`True` if the text is empty or contains only whitespace,
-              :data:`False` otherwise.
-    """
-    return len(text) == 0 or text.isspace()
-
-def format(text, *args, **kw):
-    """
-    Format a string using the string formatting operator and/or :func:`str.format()`.
-
-    :param text: The text to format (a string).
-    :param args: Any positional arguments are interpolated into the text using
-                 the string formatting operator (``%``). If no positional
-                 arguments are given no interpolation is done.
-    :param kw: Any keyword arguments are interpolated into the text using the
-               :func:`str.format()` function. If no keyword arguments are given
-               no interpolation is done.
-    :returns: The text with any positional and/or keyword arguments
-              interpolated (a string).
-    """
-    if args:
-        text %= args
-    if kw:
-        text = text.format(**kw)
-    return text
-
-def format_table(data, column_names=[], horizontal_bar='-', vertical_bar='|'):
-    """
-    Render a table using characters like dashes and vertical bars to emulate borders.
-
-    :param data: An iterable (e.g. a :class:`tuple` or :class:`list`)
-                 containing the rows of the table, where each row is an
-                 iterable containing the columns of the table (strings).
-    :param column_names: A iterable of column names (strings).
-    :param horizontal_bar: The character used to represent a horizontal bar (a
-                           string).
-    :param vertical_bar: The character used to represent a vertical bar (a
-                         string).
-    :returns: The rendered table (a string).
-    """
-    lines = []
-    # Normalize the input to a nested list of Unicode strings (this avoids
-    # problems when the caller passes in something like a generator that can
-    # only be iterated once).
-    data = [[unicode_type(column) for column in row] for row in data]
-    # Normalize the column names into a list of Unicode strings and add the
-    # list to the front of the list with table data.
-    column_names = [unicode_type(name) for name in column_names]
-    if column_names:
-        data.insert(0, column_names)
-    # Calculate the maximum width of each column.
-    widths = collections.defaultdict(int)
-    for row in data:
-        for column_index, column in enumerate(row):
-            widths[column_index] = max(widths[column_index], len(column))
-    # Create a horizontal bar of dashes as a delimiter.
-    line_delimiter = horizontal_bar * (sum(widths.values()) + len(widths) * 3 + 1)
-    lines.append(line_delimiter)
-    # Format the rows and columns.
-    for row_number, row in enumerate(data):
-        line = [vertical_bar]
-        for j, column in enumerate(row):
-            padding = ' ' * (widths[j] - len(column))
-            line.append(' ' + column + padding + ' ')
-            line.append(vertical_bar)
-        lines.append(u''.join(line))
-        if column_names and row_number == 0:
-            lines.append(line_delimiter)
-    lines.append(line_delimiter)
-    return u'\n'.join(lines)
 
 def coerce_boolean(value):
     """
@@ -207,16 +105,18 @@ def coerce_boolean(value):
     else:
         return bool(value)
 
+
 def format_size(num_bytes, keep_width=False):
     """
-    Format a byte count as a human readable file size (supports ranges from
-    kilobytes to terabytes).
+    Format a byte count as a human readable file size.
 
     :param num_bytes: The size to format in bytes (an integer).
     :param keep_width: ``True`` if trailing zeros should not be stripped,
                        ``False`` if they can be stripped.
     :returns: The corresponding human readable file size (a string).
 
+    This function supports ranges from kilobytes to terabytes.
+
     Some examples:
 
     >>> from humanfriendly import format_size
@@ -237,13 +137,14 @@ def format_size(num_bytes, keep_width=False):
             return pluralize(number, unit['singular'], unit['plural'])
     return pluralize(num_bytes, 'byte')
 
+
 def parse_size(size):
     """
-    Parse a human readable data size and return the number of bytes. Raises
-    :py:class:`InvalidSize` when the size cannot be parsed.
+    Parse a human readable data size and return the number of bytes.
 
     :param size: The human readable file size to parse (a string).
     :returns: The corresponding size in bytes (an integer).
+    :raises: :exc:`InvalidSize` when the input can't be parsed.
 
     Some examples:
 
@@ -257,28 +158,95 @@ def parse_size(size):
     >>> parse_size('1.5 GB')
     1610612736
     """
-    tokens = re.split(r'([0-9.]+)', size.lower())
-    components = [s.strip() for s in tokens if s and not s.isspace()]
-    if len(components) == 1 and components[0].isdigit():
-        # If the string contains only an integer number, it is assumed to be
-        # the number of bytes.
-        return int(components[0])
-    # Otherwise we expect to find two tokens: A number and a unit.
-    if len(components) != 2:
-        msg = "Expected to get two tokens, got %s!"
-        raise InvalidSize(msg % components)
-    # Try to match the first letter of the unit.
-    for unit in reversed(disk_size_units):
-        if components[1].startswith(unit['prefix']):
-            return int(float(components[0]) * unit['divider'])
-    # Failed to match a unit: Explain what went wrong.
-    msg = "Invalid disk size unit: %r"
-    raise InvalidSize(msg % components[1])
+    tokens = tokenize(size)
+    if tokens and isinstance(tokens[0], numbers.Number):
+        # If the input contains only a number, it's assumed to be the number of bytes.
+        if len(tokens) == 1:
+            return int(tokens[0])
+        # Otherwise we expect to find two tokens: A number and a unit.
+        if len(tokens) == 2 and isinstance(tokens[1], string_types):
+            normalized_unit = tokens[1].lower()
+            # Try to match the first letter of the unit.
+            for unit in disk_size_units:
+                if normalized_unit.startswith(unit['prefix']):
+                    return int(tokens[0] * unit['divider'])
+    # We failed to parse the size specification.
+    msg = "Failed to parse size! (input %r was tokenized as %r)"
+    raise InvalidSize(msg % (size, tokens))
+
+
+def format_length(num_metres, keep_width=False):
+    """
+    Format a metre count as a human readable length.
+
+    :param num_metres: The length to format in metres (float / integer).
+    :param keep_width: ``True`` if trailing zeros should not be stripped,
+                       ``False`` if they can be stripped.
+    :returns: The corresponding human readable length (a string).
+
+    This function supports ranges from nanometres to kilometres.
+
+    Some examples:
+
+    >>> from humanfriendly import format_length
+    >>> format_length(0)
+    '0 metres'
+    >>> format_length(1)
+    '1 metre'
+    >>> format_length(5)
+    '5 metres'
+    >>> format_length(1000)
+    '1 km'
+    >>> format_length(0.004)
+    '4 mm'
+    """
+    for unit in reversed(length_size_units):
+        if num_metres >= unit['divider']:
+            number = round_number(float(num_metres) / unit['divider'], keep_width=keep_width)
+            return pluralize(number, unit['singular'], unit['plural'])
+    return pluralize(num_metres, 'metre')
+
+
+def parse_length(length):
+    """
+    Parse a human readable length and return the number of metres.
+
+    :param length: The human readable length to parse (a string).
+    :returns: The corresponding length in metres (a float).
+    :raises: :exc:`InvalidLength` when the input can't be parsed.
+
+    Some examples:
+
+    >>> from humanfriendly import parse_length
+    >>> parse_length('42')
+    42
+    >>> parse_length('1 km')
+    1000
+    >>> parse_length('5mm')
+    0.005
+    >>> parse_length('15.3cm')
+    0.153
+    """
+    tokens = tokenize(length)
+    if tokens and isinstance(tokens[0], numbers.Number):
+        # If the input contains only a number, it's assumed to be the number of metres.
+        if len(tokens) == 1:
+            return int(tokens[0])
+        # Otherwise we expect to find two tokens: A number and a unit.
+        if len(tokens) == 2 and isinstance(tokens[1], string_types):
+            normalized_unit = tokens[1].lower()
+            # Try to match the first letter of the unit.
+            for unit in length_size_units:
+                if normalized_unit.startswith(unit['prefix']):
+                    return tokens[0] * unit['divider']
+    # We failed to parse the length specification.
+    msg = "Failed to parse length! (input %r was tokenized as %r)"
+    raise InvalidLength(msg % (length, tokens))
+
 
 def format_number(number, num_decimals=2):
     """
-    Format a number as a string including thousands separators to make it
-    easier to recognize the order of size of the number.
+    Format a number as a string including thousands separators.
 
     :param number: The number to format (a number like an :class:`int`,
                    :class:`long` or :class:`float`).
@@ -287,6 +255,9 @@ def format_number(number, num_decimals=2):
                          they will be omitted regardless of this argument.
     :returns: The formatted number (a string).
 
+    This function is intended to make it easier to recognize the order of size
+    of the number being formatted.
+
     Here's an example:
 
     >>> from humanfriendly import format_number
@@ -309,19 +280,22 @@ def format_number(number, num_decimals=2):
         formatted_number += '.' + decimals_to_add
     return formatted_number
 
+
 def round_number(count, keep_width=False):
     """
-    Helper for :py:func:`format_size()` and :py:func:`format_timespan()` to
-    round a floating point number to two decimal places in a human friendly
-    format. If no decimal places are required to represent the number, they
-    will be omitted.
+    Round a floating point number to two decimal places in a human friendly format.
 
     :param count: The number to format.
     :param keep_width: ``True`` if trailing zeros should not be stripped,
                        ``False`` if they can be stripped.
-    :returns: The formatted number as a string.
+    :returns: The formatted number as a string. If no decimal places are
+              required to represent the number, they will be omitted.
 
-    An example:
+    The main purpose of this function is to be used by functions like
+    :func:`format_length()`, :func:`format_size()` and
+    :func:`format_timespan()`.
+
+    Here are some examples:
 
     >>> from humanfriendly import round_number
     >>> round_number(1)
@@ -337,20 +311,25 @@ def round_number(count, keep_width=False):
         text = re.sub('\.$', '', text)
     return text
 
-def format_timespan(num_seconds):
+
+def format_timespan(num_seconds, detailed=False):
     """
     Format a timespan in seconds as a human readable string.
 
     :param num_seconds: Number of seconds (integer or float).
+    :param detailed: If :data:`True` milliseconds are represented separately
+                     instead of being represented as fractional seconds
+                     (defaults to :data:`False`).
     :returns: The formatted timespan as a string.
 
     Some examples:
 
     >>> from humanfriendly import format_timespan
     >>> format_timespan(0)
-    '0.00 seconds'
+    '0 seconds'
     >>> format_timespan(1)
-    '1.00 second'
+    '1 second'
+    >>> import math
     >>> format_timespan(math.pi)
     '3.14 seconds'
     >>> hour = 60 * 60
@@ -359,7 +338,7 @@ def format_timespan(num_seconds):
     >>> format_timespan(week * 52 + day * 2 + hour * 3)
     '1 year, 2 days and 3 hours'
     """
-    if num_seconds < 60:
+    if num_seconds < 60 and not detailed:
         # Fast path.
         return pluralize(round_number(num_seconds), 'second')
     else:
@@ -374,22 +353,78 @@ def format_timespan(num_seconds):
             # A single count/unit combination.
             return result[0]
         else:
-            # Remove insignificant data from the formatted timespan and format
-            # it in a readable way.
-            return concatenate(result[:3])
+            if not detailed:
+                # Remove insignificant data from the formatted timespan.
+                result = result[:3]
+            # Format the timespan in a readable way.
+            return concatenate(result)
+
+
+def parse_timespan(timespan):
+    """
+    Parse a "human friendly" timespan into the number of seconds.
+
+    :param value: A string like ``5h`` (5 hours), ``10m`` (10 minutes) or
+                  ``42s`` (42 seconds).
+    :returns: The number of seconds as a floating point number.
+    :raises: :exc:`InvalidTimespan` when the input can't be parsed.
+
+    Note that the :func:`parse_timespan()` function is not meant to be the
+    "mirror image" of the :func:`format_timespan()` function. Instead it's
+    meant to allow humans to easily and succinctly specify a timespan with a
+    minimal amount of typing. It's very useful to accept easy to write time
+    spans as e.g. command line arguments to programs.
+
+    Some examples:
+
+    >>> from humanfriendly import parse_timespan
+    >>> parse_timespan('42')
+    42.0
+    >>> parse_timespan('42s')
+    42.0
+    >>> parse_timespan('1m')
+    60.0
+    >>> parse_timespan('1h')
+    3600.0
+    >>> parse_timespan('1d')
+    86400.0
+    """
+    tokens = tokenize(timespan)
+    if tokens and isinstance(tokens[0], numbers.Number):
+        # If the input contains only a number, it's assumed to be the number of seconds.
+        if len(tokens) == 1:
+            return float(tokens[0])
+        # Otherwise we expect to find two tokens: A number and a unit.
+        if len(tokens) == 2 and isinstance(tokens[1], string_types):
+            normalized_unit = tokens[1].lower()
+            for unit in time_units:
+                if normalized_unit in (unit['singular'], unit['plural'], unit['abbreviation']):
+                    return float(tokens[0]) * unit['divider']
+    # We failed to parse the timespan specification.
+    msg = "Failed to parse timespan! (input %r was tokenized as %r)"
+    raise InvalidTimespan(msg % (timespan, tokens))
+
 
 def parse_date(datestring):
     """
-    Parse a date/time string in one of the formats listed below. Raises
-    :py:class:`InvalidDate` when the date cannot be parsed. Supported date/time
-    formats:
-
-    - ``YYYY-MM-DD``
-    - ``YYYY-MM-DD HH:MM:SS``
+    Parse a date/time string into a tuple of integers.
 
     :param datestring: The date/time string to parse.
     :returns: A tuple with the numbers ``(year, month, day, hour, minute,
               second)`` (all numbers are integers).
+    :raises: :exc:`InvalidDate` when the date cannot be parsed.
+
+    Supported date/time formats:
+
+    - ``YYYY-MM-DD``
+    - ``YYYY-MM-DD HH:MM:SS``
+
+    .. note:: If you want to parse date/time strings with a fixed, known
+              format and :func:`parse_date()` isn't useful to you, consider
+              :func:`time.strptime()` or :meth:`datetime.datetime.strptime()`,
+              both of which are included in the Python standard library.
+              Alternatively for more complex tasks consider using the date/time
+              parsing module in the dateutil_ package.
 
     Examples:
 
@@ -424,6 +459,7 @@ def parse_date(datestring):
     >>> print(format_timespan(seconds_since_then))
     1 year, 43 weeks and 1 day
 
+    .. _dateutil: https://dateutil.readthedocs.org/en/latest/parser.html
     .. _Unix time: http://en.wikipedia.org/wiki/Unix_time
     """
     try:
@@ -439,16 +475,19 @@ def parse_date(datestring):
         msg = "Invalid date! (expected 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS' but got: %r)"
         raise InvalidDate(msg % datestring)
 
+
 def format_path(pathname):
     """
-    Given an absolute pathname, abbreviate the user's home directory to ``~/``
-    in order to shorten the pathname without losing information. It is not an
-    error if the pathname is not relative to the current user's home
-    directory.
+    Shorten a pathname to make it more human friendly.
 
     :param pathname: An absolute pathname (a string).
     :returns: The pathname with the user's home directory abbreviated.
 
+    Given an absolute pathname, this function abbreviates the user's home
+    directory to ``~/`` in order to shorten the pathname without losing
+    information. It is not an error if the pathname is not relative to the
+    current user's home directory.
+
     Here's an example of its usage:
 
     >>> from os import environ
@@ -468,6 +507,7 @@ def format_path(pathname):
             pathname = os.path.join('~', os.path.relpath(pathname, home))
     return pathname
 
+
 def parse_path(pathname):
     """
     Convert a human friendly pathname to an absolute pathname.
@@ -481,42 +521,6 @@ def parse_path(pathname):
     """
     return os.path.abspath(os.path.expanduser(os.path.expandvars(pathname)))
 
-def pluralize(count, singular, plural=None):
-    """
-    Combine a count with the singular or plural form of a word.
-
-    If the plural form of the word is not provided it is obtained by
-    concatenating the singular form of the word with the letter "s". Of course
-    this will not always be correct, which is why you have the option to
-    specify both forms.
-
-    :param count: The count (a number).
-    :param singular: The singular form of the word (a string).
-    :param plural: The plural form of the word (a string or ``None``).
-    :returns: The count and singular/plural word concatenated (a string).
-    """
-    if not plural:
-        plural = singular + 's'
-    return '%s %s' % (count, singular if math.floor(float(count)) == 1 else plural)
-
-def concatenate(items):
-    """
-    Concatenate a list of items in a human friendly way.
-
-    :param items: A sequence of strings.
-    :returns: A single string.
-
-    >>> from humanfriendly import concatenate
-    >>> concatenate(["eggs", "milk", "bread"])
-    'eggs, milk and bread'
-    """
-    items = list(items)
-    if len(items) > 1:
-        return ', '.join(items[:-1]) + ' and ' + items[-1]
-    elif items:
-        return items[0]
-    else:
-        return ''
 
 def prompt_for_choice(choices, default=None):
     """
@@ -530,7 +534,7 @@ def prompt_for_choice(choices, default=None):
     # By default the raw_input() prompt is very unfriendly, for example the
     # `Home' key enters `^[OH' and the `End' key enters `^[OF'. By simply
     # importing the `readline' module the prompt becomes much friendlier.
-    import readline
+    import readline  # NOQA
     # Make sure we can use 'choices' more than once (i.e. not a generator).
     choices = list(choices)
     # Present the available choices in a user friendly way.
@@ -575,6 +579,7 @@ def prompt_for_choice(choices, default=None):
         else:
             print("Error: The string '%s' doesn't match any choices." % input)
 
+
 class Timer(object):
 
     """
@@ -630,39 +635,99 @@ class Timer(object):
         return format_timespan(round(self.elapsed_time))
 
     def __str__(self):
-        """
-        When a :py:class:`Timer` is coerced to a string it will show the
-        elapsed time since the :py:class:`Timer` was created.
-        """
+        """Show the elapsed time since the :class:`Timer` was created."""
         return format_timespan(self.elapsed_time)
 
+
 class Spinner(object):
 
     """
-    Show a "spinner" on the terminal to let the user know that something is
-    happening during long running operations that would otherwise be silent.
+    Show a spinner on the terminal as a simple means of feedback to the user.
 
-    >>> from time import sleep
-    >>> from humanfriendly import Spinner
-    >>> spin = Spinner(label="Downloading")
-    >>> for i in xrange(100):
-        sleep(0.1)
-        spin.step()
-    | Downloading  # cycles through | / - \\
-    >>> spin = Spinner(label="Downloading", total=100)
-    >>> for i in xrange(100):
-        sleep(0.1)
-        spin.step(i)
-    | Downloading: 1.00% # travels up to 100%...
+    The :class:`Spinner` class shows a "spinner" on the terminal to let the
+    user know that something is happening during long running operations that
+    would otherwise be silent (leaving the user to wonder what they're waiting
+    for). Below are some visual examples that should illustrate the point.
+
+    **Simple spinners:**
+
+     Here's a screen capture that shows the simplest form of spinner:
+
+      .. image:: images/spinner-basic.gif
+         :alt: Animated screen capture of a simple spinner.
+
+     The following code was used to create the spinner above:
+
+     .. code-block:: python
+
+        import itertools
+        import time
+        from humanfriendly import Spinner
+
+        with Spinner(label="Downloading") as spinner:
+            for i in itertools.count():
+                # Do something useful here.
+                time.sleep(0.1)
+                # Advance the spinner.
+                spinner.step()
+
+    **Spinners that show elapsed time:**
+
+     Here's a spinner that shows the elapsed time since it started:
+
+      .. image:: images/spinner-with-timer.gif
+         :alt: Animated screen capture of a spinner showing elapsed time.
+
+     The following code was used to create the spinner above:
+
+     .. code-block:: python
+
+        import itertools
+        import time
+        from humanfriendly import Spinner, Timer
+
+        with Spinner(label="Downloading", timer=Timer()) as spinner:
+            for i in itertools.count():
+                # Do something useful here.
+                time.sleep(0.1)
+                # Advance the spinner.
+                spinner.step()
+
+    **Spinners that show progress:**
+
+     Here's a spinner that shows a progress percentage:
+
+      .. image:: images/spinner-with-progress.gif
+         :alt: Animated screen capture of spinner showing progress.
+
+     The following code was used to create the spinner above:
+
+     .. code-block:: python
+
+        import itertools
+        import random
+        import time
+        from humanfriendly import Spinner, Timer
+
+        with Spinner(label="Downloading", total=100) as spinner:
+            progress = 0
+            while progress < 100:
+                # Do something useful here.
+                time.sleep(0.1)
+                # Advance the spinner.
+                spinner.step(progress)
+                # Determine the new progress value.
+                progress += random.random() * 5
 
     If you want to provide user feedback during a long running operation but
     it's not practical to periodically call the :py:func:`~Spinner.step()`
     method consider using :py:class:`AutomaticSpinner` instead.
 
-    :class:`Spinner` objects can be used as context managers to automatically
-    call :func:`clear()` when the spinner ends. This helps to make sure that if
-    the text cursor is hidden its visibility is restored before the spinner
-    ends (even if an exception interrupts the spinner).
+    As you may already have noticed in the examples above, :class:`Spinner`
+    objects can be used as context managers to automatically call
+    :func:`clear()` when the spinner ends. This helps to make sure that if the
+    text cursor is hidden its visibility is restored before the spinner ends
+    (even if an exception interrupts the spinner).
     """
 
     def __init__(self, label=None, total=0, stream=sys.stderr, interactive=None, timer=None, hide_cursor=True):
@@ -703,11 +768,26 @@ class Spinner(object):
 
     def step(self, progress=0, label=None):
         """
-        Advance the spinner by one step without starting a new line, causing
-        an animated effect which is very simple but much nicer than waiting
-        for a prompt which is completely silent for a long time. Progress
-        should be the amount out of ``Spinner.total`` that is complete, not
-        a step amount.
+        Advance the spinner by one step and redraw it.
+
+        :param progress: The number of the current step, relative to the total
+                         given to the :class:`Spinner` constructor (an integer,
+                         optional). If not provided the spinner will not show
+                         progress.
+        :param label: The label to use while redrawing (a string, optional). If
+                      not provided the label given to the :class:`Spinner`
+                      constructor is used instead.
+
+        This method advances the spinner by one step without starting a new
+        line, causing an animated effect which is very simple but much nicer
+        than waiting for a prompt which is completely silent for a long time.
+
+        .. note:: This method uses time based rate limiting to avoid redrawing
+                  the spinner too frequently. If you know you're dealing with
+                  code that will call :func:`step()` at a high frequency,
+                  consider using :func:`sleep()` to avoid creating the
+                  equivalent of a busy loop that's rate limiting the spinner
+                  99% of the time.
         """
         if self.interactive:
             time_now = time.time()
@@ -726,17 +806,26 @@ class Spinner(object):
 
     def sleep(self):
         """
-        Sleep for a short period (less than a second) before refreshing the
-        spinner so that the animated effect of the spinner works best (this
-        doesn't refresh the spinner, use :func:`step()` for that).
+        Sleep for a short period before redrawing the spinner.
+
+        This method is useful when you know you're dealing with code that will
+        call :func:`step()` at a high frequency. It will sleep for the interval
+        with which the spinner is redrawn (less than a second). This avoids
+        creating the equivalent of a busy loop that's rate limiting the
+        spinner 99% of the time.
+
+        This method doesn't redraw the spinner, you still have to call
+        :func:`step()` in order to do that.
         """
         time.sleep(minimum_spinner_interval)
 
     def clear(self):
         """
-        Clear the spinner. The next line which is shown on the standard
-        output or error stream after calling this method will overwrite the
-        line that used to show the spinner.
+        Clear the spinner.
+
+        The next line which is shown on the standard output or error stream
+        after calling this method will overwrite the line that used to show the
+        spinner. Also the visibility of the text cursor is restored.
         """
         if self.interactive:
             if self.hide_cursor:
@@ -744,18 +833,23 @@ class Spinner(object):
             self.stream.write(erase_line_code)
 
     def __enter__(self):
+        """Enable the use of spinners as context managers."""
         return self
 
     def __exit__(self, exc_type=None, exc_value=None, traceback=None):
+        """Enable the use of spinners as context managers."""
         self.clear()
 
+
 class AutomaticSpinner(object):
 
     """
-    Show a "spinner" on the terminal (just like :py:class:`Spinner` does) that
-    automatically starts animating. This class should be used as a context
-    manager using the :py:keyword:`with` statement. The animation continues for
-    as long as the context is active.
+    Show a spinner on the terminal that automatically starts animating.
+
+    This class shows a spinner on the terminal (just like :py:class:`Spinner`
+    does) that automatically starts animating. This class should be used as a
+    context manager using the :py:keyword:`with` statement. The animation
+    continues for as long as the context is active.
 
     :py:class:`AutomaticSpinner` provides an alternative to :py:class:`Spinner`
     for situations where it is not practical for the caller to periodically
@@ -778,46 +872,39 @@ class AutomaticSpinner(object):
         :param show_time: If this is ``True`` (the default) then the spinner
                           shows elapsed time.
         """
+        self.label = label
+        self.show_time = show_time
         self.shutdown_event = multiprocessing.Event()
-        self.subprocess = multiprocessing.Process(target=automatic_spinner_target,
-                                                  args=(label, show_time, self.shutdown_event))
+        self.subprocess = multiprocessing.Process(target=self._target)
 
     def __enter__(self):
+        """Enable the use of automatic spinners as context managers."""
         self.subprocess.start()
 
     def __exit__(self, exc_type=None, exc_value=None, traceback=None):
+        """Enable the use of automatic spinners as context managers."""
         self.shutdown_event.set()
         self.subprocess.join()
 
-def automatic_spinner_target(label, show_time, shutdown_event):
-    try:
-        timer = Timer() if show_time else None
-        with Spinner(label=label, timer=timer) as spinner:
-            while not shutdown_event.is_set():
-                spinner.step()
-                spinner.sleep()
-    except KeyboardInterrupt:
-        # Swallow Control-C signals without producing a nasty traceback that
-        # won't make any sense to the average user.
-        pass
+    def _target(self):
+        try:
+            timer = Timer() if self.show_time else None
+            with Spinner(label=self.label, timer=timer) as spinner:
+                while not self.shutdown_event.is_set():
+                    spinner.step()
+                    spinner.sleep()
+        except KeyboardInterrupt:
+            # Swallow Control-C signals without producing a nasty traceback that
+            # won't make any sense to the average user.
+            pass
 
-class InvalidSize(Exception):
-    """
-    Raised by :py:func:`parse_size()` when a string cannot be parsed into a
-    file size:
-
-    >>> from humanfriendly import parse_size
-    >>> parse_size('5 Z')
-    Traceback (most recent call last):
-      File "humanfriendly.py", line 98, in parse_size
-        raise InvalidSize, msg % components[1]
-    humanfriendly.InvalidSize: Invalid disk size unit: 'z'
-    """
 
 class InvalidDate(Exception):
+
     """
-    Raised by :py:func:`parse_date()` when a string cannot be parsed into a
-    date:
+    Raised when a string cannot be parsed into a date.
+
+    For example:
 
     >>> from humanfriendly import parse_date
     >>> parse_date('2013-06-XY')
@@ -826,3 +913,51 @@ class InvalidDate(Exception):
         raise InvalidDate, msg % datestring
     humanfriendly.InvalidDate: Invalid date! (expected 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS' but got: '2013-06-XY')
     """
+
+
+class InvalidSize(Exception):
+
+    """
+    Raised when a string cannot be parsed into a file size.
+
+    For example:
+
+    >>> from humanfriendly import parse_size
+    >>> parse_size('5 Z')
+    Traceback (most recent call last):
+      File "humanfriendly/__init__.py", line 267, in parse_size
+        raise InvalidSize(msg % (size, tokens))
+    humanfriendly.InvalidSize: Failed to parse size! (input '5 Z' was tokenized as [5, 'Z'])
+    """
+
+
+class InvalidLength(Exception):
+
+    """
+    Raised when a string cannot be parsed into a length.
+
+    For example:
+
+    >>> from humanfriendly import parse_length
+    >>> parse_length('5 Z')
+    Traceback (most recent call last):
+      File "humanfriendly/__init__.py", line 267, in parse_length
+        raise InvalidLength(msg % (length, tokens))
+    humanfriendly.InvalidLength: Failed to parse length! (input '5 Z' was tokenized as [5, 'Z'])
+    """
+
+
+class InvalidTimespan(Exception):
+
+    """
+    Raised when a string cannot be parsed into a timespan.
+
+    For example:
+
+    >>> from humanfriendly import parse_timespan
+    >>> parse_timespan('1 age')
+    Traceback (most recent call last):
+      File "humanfriendly/__init__.py", line 419, in parse_timespan
+        raise InvalidTimespan(msg % (timespan, tokens))
+    humanfriendly.InvalidTimespan: Failed to parse timespan! (input '1 age' was tokenized as [1, 'age'])
+    """
