@@ -1,11 +1,14 @@
 #!/usr/bin/env python
+# vim: fileencoding=utf-8 :
 
-# Tests for the 'humanfriendly' module.
+# Tests for the `humanfriendly' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: May 27, 2015
+# Last Change: September 10, 2015
 # URL: https://humanfriendly.readthedocs.org
 
+"""Test suite for the `humanfriendly` package."""
+
 # Standard library modules.
 import math
 import os
@@ -18,17 +21,27 @@ import unittest
 import humanfriendly
 import humanfriendly.cli
 from humanfriendly import compact, dedent
+from humanfriendly.tables import (
+    format_pretty_table,
+    format_robust_table,
+    format_smart_table,
+)
 from humanfriendly.terminal import (
     ANSI_CSI,
     ANSI_RESET,
     ANSI_SGR,
+    ansi_strip,
     ansi_style,
     ansi_width,
     ansi_wrap,
     connected_to_terminal,
-    find_meta_variables,
     find_terminal_size,
+)
+from humanfriendly.usage import (
+    find_meta_variables,
     format_usage,
+    import_module,
+    render_usage,
 )
 
 try:
@@ -41,7 +54,10 @@ except ImportError:
 
 class HumanFriendlyTestCase(unittest.TestCase):
 
+    """Container for the `humanfriendly` test suite."""
+
     def test_compact(self):
+        """Test :func:`humanfriendly.text.compact()`."""
         assert compact(' a \n\n b ') == 'a b'
         assert compact('''
             %s template notation
@@ -51,6 +67,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         ''', type='readable') == 'More readable template notation'
 
     def test_dedent(self):
+        """Test :func:`humanfriendly.text.dedent()`."""
         assert dedent('\n line 1\n  line 2\n\n') == 'line 1\n line 2\n'
         assert dedent('''
             Dedented, %s text
@@ -60,12 +77,14 @@ class HumanFriendlyTestCase(unittest.TestCase):
         ''', op='formatted') == 'Dedented, formatted text\n'
 
     def test_pluralization(self):
+        """Test :func:`humanfriendly.pluralize()`."""
         self.assertEqual('1 word', humanfriendly.pluralize(1, 'word'))
         self.assertEqual('2 words', humanfriendly.pluralize(2, 'word'))
         self.assertEqual('1 box', humanfriendly.pluralize(1, 'box', 'boxes'))
         self.assertEqual('2 boxes', humanfriendly.pluralize(2, 'box', 'boxes'))
 
     def test_boolean_coercion(self):
+        """Test :func:`humanfriendly.coerce_boolean()`."""
         for value in [True, 'TRUE', 'True', 'true', 'on', 'yes', '1']:
             self.assertEqual(True, humanfriendly.coerce_boolean(value))
         for value in [False, 'FALSE', 'False', 'false', 'off', 'no', '0']:
@@ -73,11 +92,15 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertRaises(ValueError, humanfriendly.coerce_boolean, 'not a boolean')
 
     def test_format_timespan(self):
+        """Test :func:`humanfriendly.format_timespan()`."""
         minute = 60
         hour = minute * 60
         day = hour * 24
         week = day * 7
         year = week * 52
+        self.assertEqual('1 millisecond', humanfriendly.format_timespan(0.001, detailed=True))
+        self.assertEqual('500 milliseconds', humanfriendly.format_timespan(0.5, detailed=True))
+        self.assertEqual('0.5 seconds', humanfriendly.format_timespan(0.5, detailed=False))
         self.assertEqual('0 seconds', humanfriendly.format_timespan(0))
         self.assertEqual('0.54 seconds', humanfriendly.format_timespan(0.54321))
         self.assertEqual('1 second', humanfriendly.format_timespan(1))
@@ -93,14 +116,41 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual('2 weeks', humanfriendly.format_timespan(week * 2))
         self.assertEqual('1 year', humanfriendly.format_timespan(year))
         self.assertEqual('2 years', humanfriendly.format_timespan(year * 2))
-        self.assertEqual('1 year, 2 weeks and 3 days', humanfriendly.format_timespan(year + week * 2 + day * 3 + hour * 12))
+        self.assertEqual('6 years, 5 weeks, 4 days, 3 hours, 2 minutes and 500 milliseconds',
+                         humanfriendly.format_timespan(year*6 + week*5 + day*4 + hour*3 + minute*2 + 0.5,
+                                                       detailed=True))
+        self.assertEqual(
+            '1 year, 2 weeks and 3 days',
+            humanfriendly.format_timespan(year + week * 2 + day * 3 + hour * 12))
+
+    def test_parse_timespan(self):
+        """Test :func:`humanfriendly.parse_timespan()`."""
+        self.assertEqual(0, humanfriendly.parse_timespan('0'))
+        self.assertEqual(0, humanfriendly.parse_timespan('0s'))
+        self.assertEqual(0.001, humanfriendly.parse_timespan('1ms'))
+        self.assertEqual(0.001, humanfriendly.parse_timespan('1 millisecond'))
+        self.assertEqual(0.5, humanfriendly.parse_timespan('500 milliseconds'))
+        self.assertEqual(0.5, humanfriendly.parse_timespan('0.5 seconds'))
+        self.assertEqual(5, humanfriendly.parse_timespan('5s'))
+        self.assertEqual(5, humanfriendly.parse_timespan('5 seconds'))
+        self.assertEqual(60*2, humanfriendly.parse_timespan('2m'))
+        self.assertEqual(60*2, humanfriendly.parse_timespan('2 minutes'))
+        self.assertEqual(60*60*3, humanfriendly.parse_timespan('3 h'))
+        self.assertEqual(60*60*3, humanfriendly.parse_timespan('3 hours'))
+        self.assertEqual(60*60*24*4, humanfriendly.parse_timespan('4d'))
+        self.assertEqual(60*60*24*4, humanfriendly.parse_timespan('4 days'))
+        self.assertEqual(60*60*24*7*5, humanfriendly.parse_timespan('5 w'))
+        self.assertEqual(60*60*24*7*5, humanfriendly.parse_timespan('5 weeks'))
+        self.assertRaises(humanfriendly.InvalidTimespan, humanfriendly.parse_timespan, '1z')
 
     def test_parse_date(self):
+        """Test :func:`humanfriendly.parse_date()`."""
         self.assertEqual((2013, 6, 17, 0, 0, 0), humanfriendly.parse_date('2013-06-17'))
         self.assertEqual((2013, 6, 17, 2, 47, 42), humanfriendly.parse_date('2013-06-17 02:47:42'))
         self.assertRaises(humanfriendly.InvalidDate, humanfriendly.parse_date, '2013-06-XY')
 
     def test_format_size(self):
+        """Test :func:`humanfriendly.format_size()`."""
         self.assertEqual('0 bytes', humanfriendly.format_size(0))
         self.assertEqual('1 byte', humanfriendly.format_size(1))
         self.assertEqual('42 bytes', humanfriendly.format_size(42))
@@ -111,6 +161,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual('1 PB', humanfriendly.format_size(1024 ** 5))
 
     def test_parse_size(self):
+        """Test :func:`humanfriendly.parse_size()`."""
         self.assertEqual(0, humanfriendly.parse_size('0B'))
         self.assertEqual(42, humanfriendly.parse_size('42'))
         self.assertEqual(42, humanfriendly.parse_size('42B'))
@@ -118,10 +169,36 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual(1024, humanfriendly.parse_size('1 KB'))
         self.assertEqual(1024, humanfriendly.parse_size('1 kilobyte'))
         self.assertEqual(1024 ** 3, humanfriendly.parse_size('1 GB'))
+        self.assertEqual(1024 ** 3 * 1.5, humanfriendly.parse_size('1.5 GB'))
         self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, '1z')
         self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, 'a')
 
+    def test_format_length(self):
+        """Test :func:`humanfriendly.format_length()`."""
+        self.assertEqual('0 metres', humanfriendly.format_length(0))
+        self.assertEqual('1 metre', humanfriendly.format_length(1))
+        self.assertEqual('42 metres', humanfriendly.format_length(42))
+        self.assertEqual('1 km', humanfriendly.format_length(1 * 1000))
+        self.assertEqual('15.3 cm', humanfriendly.format_length(0.153))
+        self.assertEqual('1 cm', humanfriendly.format_length(1e-02))
+        self.assertEqual('1 mm', humanfriendly.format_length(1e-03))
+        self.assertEqual('1 nm', humanfriendly.format_length(1e-09))
+
+    def test_parse_length(self):
+        """Test :func:`humanfriendly.parse_length()`."""
+        self.assertEqual(0, humanfriendly.parse_length('0m'))
+        self.assertEqual(42, humanfriendly.parse_length('42'))
+        self.assertEqual(42, humanfriendly.parse_length('42m'))
+        self.assertEqual(1000, humanfriendly.parse_length('1km'))
+        self.assertEqual(0.153, humanfriendly.parse_length('15.3 cm'))
+        self.assertEqual(1e-02, humanfriendly.parse_length('1cm'))
+        self.assertEqual(1e-03, humanfriendly.parse_length('1mm'))
+        self.assertEqual(1e-09, humanfriendly.parse_length('1nm'))
+        self.assertRaises(humanfriendly.InvalidLength, humanfriendly.parse_length, '1z')
+        self.assertRaises(humanfriendly.InvalidLength, humanfriendly.parse_length, 'a')
+
     def test_format_number(self):
+        """Test :func:`humanfriendly.format_number()`."""
         self.assertEqual('1', humanfriendly.format_number(1))
         self.assertEqual('1.5', humanfriendly.format_number(1.5))
         self.assertEqual('1.56', humanfriendly.format_number(1.56789))
@@ -132,38 +209,45 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual('1,000,000.42', humanfriendly.format_number(1000000.42))
 
     def test_round_number(self):
+        """Test :func:`humanfriendly.round_number()`."""
         self.assertEqual('1', humanfriendly.round_number(1))
         self.assertEqual('1', humanfriendly.round_number(1.0))
         self.assertEqual('1.00', humanfriendly.round_number(1, keep_width=True))
         self.assertEqual('3.14', humanfriendly.round_number(3.141592653589793))
 
     def test_format_path(self):
+        """Test :func:`humanfriendly.format_path()`."""
         friendly_path = os.path.join('~', '.vimrc')
         absolute_path = os.path.join(os.environ['HOME'], '.vimrc')
         self.assertEqual(friendly_path, humanfriendly.format_path(absolute_path))
 
     def test_parse_path(self):
+        """Test :func:`humanfriendly.parse_path()`."""
         friendly_path = os.path.join('~', '.vimrc')
         absolute_path = os.path.join(os.environ['HOME'], '.vimrc')
         self.assertEqual(absolute_path, humanfriendly.parse_path(friendly_path))
 
-    def test_table_formatting(self):
+    def test_pretty_tables(self):
+        """Test :func:`humanfriendly.tables.format_pretty_table()`."""
+        # The simplest case possible :-).
         data = [['Just one column']]
-        assert humanfriendly.format_table(data) == dedent("""
+        assert format_pretty_table(data) == dedent("""
             -------------------
             | Just one column |
             -------------------
         """).strip()
+        # A bit more complex: two rows, three columns, varying widths.
         data = [['One', 'Two', 'Three'], ['1', '2', '3']]
-        assert humanfriendly.format_table(data) == dedent("""
+        assert format_pretty_table(data) == dedent("""
             ---------------------
             | One | Two | Three |
             | 1   | 2   | 3     |
             ---------------------
         """).strip()
+        # A table including column names.
         column_names = ['One', 'Two', 'Three']
         data = [['1', '2', '3'], ['a', 'b', 'c']]
-        assert humanfriendly.format_table(data, column_names) == dedent("""
+        assert ansi_strip(format_pretty_table(data, column_names)) == dedent("""
             ---------------------
             | One | Two | Three |
             ---------------------
@@ -171,14 +255,95 @@ class HumanFriendlyTestCase(unittest.TestCase):
             | a   | b   | c     |
             ---------------------
         """).strip()
+        # A table that contains a column with only numeric data (will be right aligned).
+        column_names = ['Just a label', 'Important numbers']
+        data = [['Row one', '15'], ['Row two', '300']]
+        assert ansi_strip(format_pretty_table(data, column_names)) == dedent("""
+            ------------------------------------
+            | Just a label | Important numbers |
+            ------------------------------------
+            | Row one      |                15 |
+            | Row two      |               300 |
+            ------------------------------------
+        """).strip()
+
+    def test_robust_tables(self):
+        """Test :func:`humanfriendly.tables.format_robust_table()`."""
+        column_names = ['One', 'Two', 'Three']
+        data = [['1', '2', '3'], ['a', 'b', 'c']]
+        assert ansi_strip(format_robust_table(data, column_names)) == dedent("""
+            --------
+            One: 1
+            Two: 2
+            Three: 3
+            --------
+            One: a
+            Two: b
+            Three: c
+            --------
+        """).strip()
+        column_names = ['One', 'Two', 'Three']
+        data = [['1', '2', '3'], ['a', 'b', 'Here comes a\nmulti line column!']]
+        assert ansi_strip(format_robust_table(data, column_names)) == dedent("""
+            ------------------
+            One: 1
+            Two: 2
+            Three: 3
+            ------------------
+            One: a
+            Two: b
+            Three:
+            Here comes a
+            multi line column!
+            ------------------
+        """).strip()
+
+    def test_smart_tables(self):
+        """Test :func:`humanfriendly.tables.format_smart_table()`."""
+        column_names = ['One', 'Two', 'Three']
+        data = [['1', '2', '3'], ['a', 'b', 'c']]
+        assert ansi_strip(format_smart_table(data, column_names)) == dedent("""
+            ---------------------
+            | One | Two | Three |
+            ---------------------
+            | 1   | 2   | 3     |
+            | a   | b   | c     |
+            ---------------------
+        """).strip()
+        column_names = ['One', 'Two', 'Three']
+        data = [['1', '2', '3'], ['a', 'b', 'Here comes a\nmulti line column!']]
+        assert ansi_strip(format_smart_table(data, column_names)) == dedent("""
+            ------------------
+            One: 1
+            Two: 2
+            Three: 3
+            ------------------
+            One: a
+            Two: b
+            Three:
+            Here comes a
+            multi line column!
+            ------------------
+        """).strip()
 
     def test_concatenate(self):
+        """Test :func:`humanfriendly.concatenate()`."""
         self.assertEqual(humanfriendly.concatenate([]), '')
         self.assertEqual(humanfriendly.concatenate(['one']), 'one')
         self.assertEqual(humanfriendly.concatenate(['one', 'two']), 'one and two')
         self.assertEqual(humanfriendly.concatenate(['one', 'two', 'three']), 'one, two and three')
 
+    def test_split(self):
+        """Test :func:`humanfriendly.text.split()`."""
+        from humanfriendly.text import split
+        self.assertEqual(split(''), [])
+        self.assertEqual(split('foo'), ['foo'])
+        self.assertEqual(split('foo, bar'), ['foo', 'bar'])
+        self.assertEqual(split('foo, bar, baz'), ['foo', 'bar', 'baz'])
+        self.assertEqual(split('foo,bar,baz'), ['foo', 'bar', 'baz'])
+
     def test_timer(self):
+        """Test :func:`humanfriendly.Timer`."""
         for seconds, text in ((1, '1 second'),
                               (2, '2 seconds'),
                               (60, '1 minute'),
@@ -198,15 +363,22 @@ class HumanFriendlyTestCase(unittest.TestCase):
         # Test automatic timer.
         automatic_timer = humanfriendly.Timer()
         time.sleep(1)
-        self.assertEqual(normalize_timestamp(humanfriendly.round_number(automatic_timer.elapsed_time, keep_width=True)), '1.00')
+        self.assertEqual(normalize_timestamp(humanfriendly.round_number(
+            automatic_timer.elapsed_time,
+            keep_width=True,
+        )), '1.00')
         # Test resumable timer.
         resumable_timer = humanfriendly.Timer(resumable=True)
         for i in range(2):
             with resumable_timer:
                 time.sleep(1)
-        self.assertEqual(normalize_timestamp(humanfriendly.round_number(resumable_timer.elapsed_time, keep_width=True)), '2.00')
+        self.assertEqual(normalize_timestamp(humanfriendly.round_number(
+            resumable_timer.elapsed_time,
+            keep_width=True,
+        )), '2.00')
 
     def test_spinner(self):
+        """Test :func:`humanfriendly.Spinner`."""
         stream = StringIO()
         spinner = humanfriendly.Spinner('test spinner', total=4, stream=stream, interactive=True)
         for progress in [1, 2, 3, 4]:
@@ -223,15 +395,20 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertEqual(sorted(set(lines)), sorted(lines))
 
     def test_automatic_spinner(self):
-        # There's not a lot to test about the AutomaticSpinner class, but by at
-        # least running it here we are assured that the code functions on all
-        # supported Python versions. AutomaticSpinner is built on top of the
-        # Spinner class so at least we also have the tests for the Spinner
-        # class to back us up.
+        """
+        Test :func:`humanfriendly.AutomaticSpinner`.
+
+        There's not a lot to test about the :class:`.AutomaticSpinner` class,
+        but by at least running it here we are assured that the code functions
+        on all supported Python versions. :class:`.AutomaticSpinner` is built
+        on top of the :class:`.Spinner` class so at least we also have the
+        tests for the :class:`.Spinner` class to back us up.
+        """
         with humanfriendly.AutomaticSpinner('test spinner'):
             time.sleep(1)
 
     def test_prompt_for_choice(self):
+        """Test :func:`humanfriendly.prompt_for_choice`."""
         interactive_prompt = humanfriendly.interactive_prompt
         try:
             # Choice selection by full string match.
@@ -262,6 +439,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
             humanfriendly.interactive_prompt = interactive_prompt
 
     def test_cli(self):
+        """Test the command line interface."""
         # Test that the usage message is printed by default.
         returncode, output = main()
         assert 'Usage:' in output
@@ -299,6 +477,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         assert returncode == 42
 
     def test_ansi_style(self):
+        """Test :func:`humanfriendly.terminal.ansi_style()`."""
         assert ansi_style(bold=True) == '%s1%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(faint=True) == '%s2%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(underline=True) == '%s4%s' % (ANSI_CSI, ANSI_SGR)
@@ -308,6 +487,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         self.assertRaises(ValueError, ansi_style, color='unknown')
 
     def test_ansi_width(self):
+        """Test :func:`humanfriendly.terminal.ansi_width()`."""
         text = "Whatever"
         # Make sure ansi_width() works as expected on strings without ANSI escape sequences.
         assert len(text) == ansi_width(text)
@@ -321,6 +501,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         assert len(text) == ansi_width(wrapped)
 
     def test_ansi_wrap(self):
+        """Test :func:`humanfriendly.terminal.ansi_wrap()`."""
         text = "Whatever"
         # Make sure ansi_wrap() does nothing when no keyword arguments are given.
         assert text == ansi_wrap(text)
@@ -330,6 +511,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         assert ansi_wrap(text, bold=True).endswith(ANSI_RESET)
 
     def test_find_terminal_size(self):
+        """Test :func:`humanfriendly.terminal.find_terminal_size()`."""
         lines, columns = find_terminal_size()
         # We really can't assert any minimum or maximum values here because it
         # simply doesn't make any sense; it's impossible for me to anticipate
@@ -371,6 +553,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
             sys.stderr = saved_stderr
 
     def test_connected_to_terminal(self):
+        """Test :func:`humanfriendly.terminal.connected_to_terminal()`."""
         for stream in [sys.stdin, sys.stdout, sys.stderr]:
             result = connected_to_terminal(stream)
             # We really can't assert a True or False value here because this
@@ -384,6 +567,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         assert not connected_to_terminal(object())
 
     def test_find_meta_variables(self):
+        """Test :func:`humanfriendly.usage.find_meta_variables()`."""
         assert sorted(find_meta_variables("""
             Here's one example: --format-number=VALUE
             Here's another example: --format-size=BYTES
@@ -392,6 +576,7 @@ class HumanFriendlyTestCase(unittest.TestCase):
         """)) == sorted(['VALUE', 'BYTES', 'SECONDS'])
 
     def test_format_usage(self):
+        """Test :func:`humanfriendly.usage.format_usage()`."""
         # Test that options are highlighted.
         usage_text = "Just one --option"
         formatted_text = format_usage(usage_text)
@@ -416,8 +601,34 @@ class HumanFriendlyTestCase(unittest.TestCase):
         # Make sure the meta variable in the third line isn't highlighted.
         assert ANSI_CSI not in formatted_lines[2]
 
+    def test_render_usage(self):
+        """Test :func:`humanfriendly.usage.render_usage()`."""
+        assert render_usage("Usage: some-command WITH ARGS") == "**Usage:** `some-command WITH ARGS`"
+        assert render_usage("Supported options:") == "**Supported options:**"
+        assert 'code-block' in render_usage(dedent("""
+            Here comes a shell command:
+
+              $ echo test
+              test
+        """))
+        assert all(token in render_usage("""
+            Supported options:
+
+              -n, --dry-run
+
+                Don't change anything.
+        """) for token in ('`-n`', '`--dry-run`'))
+
+    def test_import_module(self):
+        """Test :func:`humanfriendly.usage.import_module()`."""
+        import humanfriendly
+        assert humanfriendly is import_module('humanfriendly')
+        from humanfriendly import cli
+        assert cli is import_module('humanfriendly.cli')
+
 
 def main(*args, **kw):
+    """Utility function to test the command line interface without invoking a subprocess."""
     returncode = 0
     input_buffer = StringIO(kw.get('input', ''))
     output_buffer = StringIO()
@@ -439,6 +650,12 @@ def main(*args, **kw):
 
 
 def normalize_timestamp(value, ndigits=1):
+    """
+    Utility function to round timestamps to the given number of digits.
+
+    This helps to make the test suite less sensitive to timing issues caused by
+    multitasking, processor scheduling, etc.
+    """
     return '%.2f' % round(float(value), ndigits=ndigits)
 
 
